/******************************************************************************
 *  Filename:       adc.c
 *  Date:           03/17/13
 *  File Verison:   1.1
 *
 *  Author:         Amira Ahsan,
 *                  Prarthana Ranjit
 *  Team:           LFEV'13
 *
 * ****************************************************************************
 *  Architecture:   Enhanced Midrange PIC
 *  Processor:      PIC16LF1827
 *  Compiler:       MPLAB X XC8
 *
 *
 * ****************************************************************************
 *  Description:    1.Sets up adc conversion from 4 different channels
 *                  Sampling rate  for each channel is 1 sample per 4 s?  was ms now?
 *                  
 *
 *
 * ****************************************************************************
 *  Pin Assignments: RB0= LED to signal power
 *                   RB3= LED controlled by the conversion results of AN1
 *                   RB5= LED toggled at 1/2 Hz
 *
 *******************************************************************************/

#include <xc.h>
#include <math.h>
#include <stdint.h>


/* configuring the bits*/
#pragma config MCLRE= OFF        //MCLR/VPP pin function is digital input
#pragma config FOSC = INTOSC     // Selects the internal oscillator
#pragma config WDTE = OFF        //Turns off the Watchdog Timer
#pragma config LVP = OFF         //Low-voltage programming disabled (high voltage on MCLR must be used for programming)
#pragma config PLLEN= OFF


#define TOFFSET (256-250+3)      //timer0 offset such that it overflows
                                 //after 250 cycles - 125 microsecond

/*functions to set the channel and start a conversion*/
#define MacroChanSel(chan)(ADCON0 = chan)          //sets up the next conversion
#define MacroStartConv()(ADCON0bits.GO=1)          //start the conversion



/*I2C transmission and receiving Buffer Allocation*/

#define MAX_TRANS_BUFF_SIZE 6 //transmission
#define MAX_REC_BUFF_SIZE 4   //receiving

unsigned char txByte[MAX_TRANS_BUFF_SIZE]; // array for bytes to be transmitted
unsigned char rxByte[MAX_REC_BUFF_SIZE];   //array for received bytes
unsigned char flush;
volatile int i =0;
volatile int it = 0;
volatile unsigned char i2c_complete =0;  //I2C command complete indicator

/*Channel selection for ADC conversions*/

#define NUM_ADC_CHAN 4
unsigned char channel_select[]=     //0:CHS<4:0>:GO_nDone:ADON------ GO_nDone=0; ADON=1
    {
        0b00000001,                 //0:AN0:GO_nDone:ADON
        0b00000101,                 //0:AN1:GO_nDone:ADON
        0b00001001,                 //0:AN2:GO_nDone:ADON
        0b00001101                  //0:AN3:GO_nDone:ADON
    };

unsigned int adc_result[NUM_ADC_CHAN];         //array to store the adc conversion results


/*Memory allocation*/
unsigned int mem=0;

//the interupt service routine
void interrupt isr(void){

    static uint16_t cnt_t0=0;       // to count number of overflows
    static unsigned int channel=0;  // used to index through the channel_select array

    unsigned int adresh_read;
    unsigned int adresl_read;

//the i2c interrupt
    if(PIR1bits.SSP1IF){            //If I2C interrupt flag is set

        PIR1bits.SSP1IF = 0;        //clear i2c interrupt flag

        if(SSP1STATbits.R_nW == 1){          //If a read command
            i = 0;
            flush = SSP1BUF;                 //if read flush the buffer

            //if(rxByte[1] < 0x04){            //given its a valid command

             //send the requested stuff to master
//                txByte[0] = rxByte[0];        //set transmission buffer
//                txByte[1] = rxByte[1];        // the command number
//                txByte[2] = rxByte[2];
//                txByte[3] = rxByte[3];
                //txByte[0] = ((unsigned char)(((adc_result[1])>>8)& 0x0003));      //High Byte of conversion of AN1
                //txByte[1] = ((unsigned char)((0x00FF & (adc_result[1]))));         //Low Byte of AN1
                txByte[0] = 0x12;          //Slave Address
                txByte[1] = 0x34;          //Command number
                txByte[2] = 0x56;          //Slave Address
                txByte[3] = 0x78;

             
            if(SSP1CON1bits.CKP == 0){        //if i2c clock is being held low

                SSP1BUF = txByte[it];         //write to i2c buffer
                SSP1CON1bits.CKP = 1;         //release i2c clock line
                it++;
            }

          //interrupt following the transfer of the last byte
             if(SSP1CON2bits.ACKSTAT == 1){

                it=0;
            }

        }else{                                //a write command

            rxByte[i] = SSP1BUF;              //read i2c buffer
            i++;
            //if a complete command has been received
            if((i)==4){
                i = 0;
                i2c_complete = 1;             //signifies that full i2c command was received
            }
        }
     }

    //the timer interrupt

    if(INTCONbits.TMR0IF==1){      //if the timer0 interrupt flag has been set

        TMR0+=TOFFSET;             //offset the Timer0 register
        INTCONbits.TMR0IF=0;       //clear the interrupt flag

        cnt_t0++;                  //increase the counter

            if(cnt_t0 == 8000){                         //cnt_0= (1s/125u)
               

                //obtain the results of a/d conversion and store it in the adc_result array
                
                adresh_read= (unsigned int)ADRESH;
                adresl_read= (unsigned int)ADRESL;
                adc_result[channel] = ((adresh_read) << 8) | adresl_read;

                //set up the next conversion
                channel++;
                //reset the index, channel if it has been increased to 4
                if(channel==NUM_ADC_CHAN){
                    channel=0;
                }
                //select the channel
                MacroChanSel(channel_select[channel]);

                //clear the ADC interrupt flag
                PIR1bits.ADIF = 0;                      // clear the adc interrupt flag
                cnt_t0=0;                               //reset  the counter
                LATBbits.LATB5 = ~(LATBbits.LATB5);     //toogle the LED
                MacroStartConv();

                     
                 }
    }

}



void main(){

  _delay(1000);                    //delays for 10 cycles
  OSCCON = 0x72;                   // Switch to fast internal 8 MHz RC Clock

 // Set up port A
  LATA   = 0x00;                   // Pre-clear any outputs
  TRISA= 0xFF  ;                    //set PortA as all inputs

  //Port Configuration for ADC conversion
  ANSELA= 0b0001111;                 //ANSA0-ANSA4-set as analog inputs

  // Set up portB
  LATB   = 0x00;                   // Pre-clear any outputs
  TRISB  = 0b00010010;             //set SDA1(RB1) and SCK1(RB4) as inputs

  /****************************************************************************/
  /*Set up I2C control registers*/

   SSP1STATbits.SMP = 0 ;          //bit 7 = SMP
   SSP1STATbits.CKE = 1;           //bit 6 = CKE
   SSP1STATbits.BF = 0;            //initially clear the buffer

   //SSP1 Control register1
   SSP1CON1bits.SSPEN = 1;         //enable the serial port and configure the SDA1and
                                   //SCL1 pins
   SSP1CON1bits.CKP = 1;           //releases the clock
   SSP1CON1bits.SSPM= 0b0110;      //I2C Slave mode, 7-bit address

   //SSP1 Control register2
   SSP1CON2bits.GCEN = 0;          //General Call disabled
   SSP1CON2bits.SEN =  0;          /*Clock Stretch for both slave transmit
                                     and receive disabled*/

   //SSP1 Control register3
   SSP1CON3bits.BOEN = 0;         //SSP1BUF is only updated when SSP1OV is clear
   SSP1CON3bits.SDAHT =0;         // 100ns minimum hold time on the SDA1 line
   SSP1CON3bits.SBCDE =0;         //bus collision detection disabled
   SSP1CON3bits.AHEN = 0;         //address holding disabled
   SSP1CON3bits.DHEN = 0;         //data holding disabled

   //SSP1 MAsk register
   SSP1MSK= 0xFF;

   //Slave ID
   SSP1ADD = 0x04;  //Only the upper 7 bits are used for the address.Default value of 0x04

/*******************************************************************************/
  /*Configure timer0*/
  OPTION_REGbits.TMR0CS = 0;        //Timer0 clock source- internal instruction cycle clock (FOSC/4)
  OPTION_REGbits.PSA = 1;           //Timer0 Prescale assignment bit- no prescale assigned
/*******************************************************************************/
  /*Configure the ADC module*/

  //Enabling Fixed Voltage Reference Selection
  FVRCONbits.FVREN=1;                 //Fixed voltage reference(FVR) is enabled
  FVRCONbits.ADFVR=0b10;              //ADC Fixed Voltage Reference Peripheral output is(2.048V)
  
  //Setting the ADC Voltage ref as the FVR
  ADCON1bits.ADNREF=0;                //Vref- is connected to Vss
  ADCON1bits.ADPREF=0b11;             //Vref+ is connectede to the internal FVR module

  //ADC conversion clock source
  ADCON1bits.ADCS= 0b111;             //Frc clock supplied from the dedicated RC oscillator

  //ADC result formatting
  ADCON1bits.ADFM=1;                  //Right justified.


  /*****************************************************************************/


// clear interrupt flags
  INTCONbits.TMR0IF=0;               // the timer interrupt flag
  PIR1bits.SSP1IF = 0;               // the i2c interrupt flag
  PIR1bits.ADIF = 0;                 // the adc interrupt flag

//enable interrupts
  
  INTCONbits.PEIE= 1;                //enable peripheral interrupt
  PIE1bits.SSP1IE = 1;               //enable i2c interrupt
  PIE1bits.ADIE=0;                   //disable ADC interrupts
  INTCONbits.TMR0IE= 1;              //enable timer0 interrupt

  ei();                              //enable global interrupts


 /*Initialize the adc_result array*/
  adc_result[0]=0x1234;
  adc_result[1]=0x5678;
  adc_result[2]=0x9abc;
  adc_result[3]=0xdeaa;



// start the first A/D conversion
  MacroChanSel(channel_select[0]);
  MacroStartConv();

  

  while(1){

      LATBbits.LATB0=1;

      mem = adc_result[1];

      //use result from channel AN1 to control LED
      if (mem>=0x0320){
          LATBbits.LATB3=1;
      }else{
          LATBbits.LATB3=0;

      }


      }

  }

