/******************************************************************************
 *  Filename:       firmware.c
 *  Date:           03/17/13
 *  File Verison:   1.1
 *
 *  Author:         Amira Ahsan,
 *                  Prarthana Ranjit
 *  Team:           LFEV'13
 *
 * ****************************************************************************
 *  Architecture:   Enhanced Midrange PIC
 *  Processor:      PIC16LF1827
 *  Compiler:       MPLAB X XC8
 *
 *
 * ****************************************************************************
 *  Description:    Firmware for the slave pic
 *
 *
 *
 * ****************************************************************************
 *  Pin Assignments:
 *                   LEDS:
 *                   RA6= PLED
 *                   RA7= WDT LED (not implemented on the BMS board)
 *                   RB0= Bypass LED
 *
 *                   ANALOG INPUT CHANNELS:
 *                   RA0/AN1=  Current_diff_dis
 *                   RA1/AN0=  Current_diff_chg
 *                   RB2/AN10= Temp
 *                   RB3/AN9=  Volt
 *
 *******************************************************************************/

#include <xc.h>
#include <math.h>
#include <stdint.h>


__EEPROM_DATA(0x04,0x14,0,0,0,0,0,0); //address default 04, and bTime default 0x04
__EEPROM_DATA(0,0,0,0,0,0,0,0);      //configure more EEPROM bytes



/* configuring the bits*/
#pragma config MCLRE= ON        //MCLR/VPP pin function is digital input
#pragma config FOSC = INTOSC     // Selects the internal oscillator
#pragma config WDTE = ON         //WDT controlled by SWDTEN

#pragma config LVP = OFF         //Low-voltage programming disabled
#pragma config PLLEN= OFF
#pragma config CPD=OFF           //external EEPROM writing permitted



#define TOFFSET (256-250+3)      //timer0 offset such that it overflows
                                 //after 250 cycles - 125 microsecond

/*functions to set the channel and start a conversion*/
#define MacroChanSel(chan)(ADCON0 = chan)          //sets up the next conversion
#define MacroStartConv()(ADCON0bits.GO=1)          //start the conversion

/*Functions supported by the slave*/

void getVolt(unsigned char x, unsigned char y);             // I2C Command# 0x10
void getTemp(unsigned char x, unsigned char y);             // I2C Command# 0x11
void getCCurr(unsigned char x, unsigned char y);            // I2C Command# 0x12
void getDCurr(unsigned char x, unsigned char y);            // I2C Command# 0x13
void getBState(unsigned char x, unsigned char y);           // I2C Command# 0x14
void getID(unsigned char x, unsigned char y);		    // I2C Command# 0x15
void getSoftware_Ver(unsigned char x, unsigned char y);     // I2C Command# 0x16
void getHex_42(unsigned char x, unsigned char y);           // I2C Command# 0x17
void getBTime(unsigned char x, unsigned char y);            // I2C Command# 0x18
void getCharge_CCount(unsigned char x, unsigned char y);    // I2C Command# 0x19
void getDischarge_CCount(unsigned char x, unsigned char y); // I2C Command# 0x1A

void getVolt_Temp(unsigned char x, unsigned char y);        // I2C Command# 0x1B
void getVolt_Temp_CCurr(unsigned char x, unsigned char y);  // I2C Command# 0x1C
void getVolt_Temp_DCurr(unsigned char x, unsigned char y);  // I2C Command# 0x1D
void getBTime_elapsed(unsigned char x, unsigned char y);    // I2C Command# 0x1E

void setBState(unsigned char x, unsigned char y);           // I2C Command# 0x00
void setID(unsigned char x, unsigned char y);               // I2C Command# 0x01
void setBTime(unsigned char x, unsigned char y);            // I2C Command# 0x02

//command to start a loop in the I2C interrupt so that WDT time is out
void startLoop(unsigned char x,unsigned char y);            //I2C Command# 0x03
                                                            //Test command for Watchdog

//function to load the transmission buffer

//mask the upper 4 bits of the command number(get-1/set-0) and take the lower 4 bits
void loadTransBuff(unsigned int command_lower_4);


/*Array of pointers to Functions*/
void (*func_ptr[4])(unsigned char x, unsigned char y)=
{
  setBState,
  setID,
  setBTime,
  
  startLoop

};
unsigned char func_ptr_index;

/*I2C transmission and receiving Buffer Allocation*/

#define MAX_TRANS_BUFF_SIZE 8
#define MAX_REC_BUFF_SIZE 4

unsigned char txByte[MAX_TRANS_BUFF_SIZE];   //array to store the transmission bytes
unsigned char rxByte[MAX_REC_BUFF_SIZE];     //array to store bytes received
unsigned char flush;
volatile int i =0;
volatile int it = 0;
volatile unsigned char i2c_complete =0;      //I2C command complete indicator
/*Isolating the lower 4 bits of the command number*/
unsigned int command_lower_4;


/*Channel selection for ADC conversions*/
/* ADC using FVR(vref)=2.048 V */
/*ADC is to be Right Justified*/

#define NUM_ADC_CHAN 4
unsigned char channel_select[]=     //0:CHS<4:0>:GO_nDone:ADON
    {
        0b00100101,                 //0:AN9:GO_nDone:ADON volt
        0b00101001,                 //0:AN10:GO_nDone:ADON temp
        0b00000101,                 //0:AN1:GO_nDone:ADON c_curr
        0b00000001                  //0:AN0:GO_nDone:ADON d_curr
    };

unsigned int adc_result[NUM_ADC_CHAN];         //array to store the adc conversion results


/*Memory allocation*/
#define MEMORY_SLOTS 11
unsigned int memory[MEMORY_SLOTS];
uint32_t charge_coulomb_count;
uint32_t discharge_coulomb_count;

//counts to keep track of the number of current reading between two consequetive coloumb count readings
volatile uint32_t no_charge_current_reading = 0;
volatile uint32_t no_discharge_current_reading = 0;


//coulomb count sums 
//to remember counts for upto 4 hours.
uint32_t charge_coulomb_sum;
uint32_t discharge_coulomb_sum;

unsigned int loop_start;

//timer to keep track of bypass time
volatile uint16_t tick_ms = 0;        // to count number of millisecond
volatile uint16_t tick_s = 0;         // to count number of seconds
volatile uint16_t tick_min = 0;       // to count number of minutes


unsigned int software_version= 0x0102;

//the timer interupt service routine
void interrupt isr(void){
    static uint16_t cnt_t0=0;           // to count number of overflows
    static uint16_t cnt_tp=0;           //count for the PLED pulse
   

    // adc conversion
    static int channel=0;          // used to index through the channel_select array
    unsigned int adresh_read;      //read the high 8 bit register of ADC result
    unsigned int adresl_read;      //read the low 8 bit register of ADC result

    //coulomb counting
     uint32_t charge_nreading;
     uint32_t discharge_nreading;
     unsigned char adr_02,adr_03,adr_04,adr_05;
     unsigned char adr_06,adr_07,adr_08,adr_09;

//the i2c interrupt
    if(PIR1bits.SSP1IF){            //If I2C interrupt flag is set

        PIR1bits.SSP1IF = 0;        //clear i2c interrupt flag

        if(SSP1STATbits.R_nW == 1){          //If a read command
            i = 0;
            flush = SSP1BUF;                 //if read flush the buffer because we don't care about the Data High and Low
            
            //load data for transmission
            command_lower_4= ((unsigned int)((rxByte[1])& 0x0F));

            //load data based on command number
            if (it == 0)loadTransBuff(command_lower_4);

            if(SSP1CON1bits.CKP == 0){        //if i2c clock is being held low

                SSP1BUF = txByte[it];         //write to i2c buffer
                SSP1CON1bits.CKP = 1;         //release i2c clock line
                it++;
            }

          //interrupt following the transfer of the last bit
             if(SSP1CON2bits.ACKSTAT == 1){
                it=0;
                //if the coulomb count was taken, clear the counts
                if ((command_lower_4==0x0009)|(command_lower_4==0x000A)){
                    //before clearing the counts add it to the sums
                    //the sums clear after 4 hours, can be extended to store more, by using more EEPROM bytes
                    charge_coulomb_sum = charge_coulomb_sum + charge_coulomb_count;
                    discharge_coulomb_sum = discharge_coulomb_sum + discharge_coulomb_count;

                    /*write into EEPROM:
                     * address 0x02,0x03,0x04,0x05 for charging coulomb count sum
                     * address 0x06,0x07,0x08,0x09 for discharging coulomb count sum
                     */
                    //charge
                adr_02= ((unsigned char)(charge_coulomb_sum>>24)& 0x000000FF);
                adr_03  = ((unsigned char)(charge_coulomb_sum>>16)& 0x000000FF);
                adr_04=  ((unsigned char)(charge_coulomb_sum>>8) & 0x000000FF);
                adr_05 = ((unsigned char)((0x000000FF & charge_coulomb_sum )));

                //discharge
                adr_06= ((unsigned char)(discharge_coulomb_sum>>24)& 0x000000FF);
                adr_07= ((unsigned char)(discharge_coulomb_sum>>16)& 0x000000FF);
                adr_08=  ((unsigned char)(discharge_coulomb_sum>>8) & 0x000000FF);
                adr_09 = ((unsigned char)((0x000000FF & discharge_coulomb_sum )));

                    EEPROM_WRITE(0x02,adr_02);
                    EEPROM_WRITE(0x03,adr_03);
                    EEPROM_WRITE(0x04,adr_04);
                    EEPROM_WRITE(0x05,adr_05);

                    EEPROM_WRITE(0x06,adr_06);
                    EEPROM_WRITE(0x07,adr_07);
                    EEPROM_WRITE(0x08,adr_08);
                    EEPROM_WRITE(0x09,adr_09);
                    while (WR) continue;


                    //clear counts
                    charge_coulomb_count= 0x00000000;
                    discharge_coulomb_count= 0x00000000;
                    no_charge_current_reading=0x00000000;
                    no_discharge_current_reading=0x00000000;
                    
                }
                LATAbits.LATA7=0;

                //loop_starter
                if (loop_start==1)
                {
                  for(;;){
                  LATAbits.LATA7=1;  //WDT count is increasing.
                                    //LED turns off as soon As WDT overflows and resets the system
                   }
                }
             }
            //clear internal watch dog
             CLRWDT();
            

        }else{                                //a write command

            rxByte[i] = SSP1BUF;              //read i2c buffer
            i++;
            //if a complete command has been received
            if((i)==4){
                i = 0;
                i2c_complete = 1;             //signifies that full i2c command was received
            }
        }
        SSP1CON1bits.CKP = 1;       // (NAING)release the clock since process finished

     }

    //the timer interrupt

    if(INTCONbits.TMR0IF==1){      //if the timer0 interrupt flag has been set

        TMR0+=TOFFSET;             //offset the Timer0 register
        INTCONbits.TMR0IF=0;       //clear the interrupt flag

        cnt_t0++;                  //increase the counter


            if(cnt_t0 == 8){                              //cnt_0= (1ms/125u)
                //obtain the results of a/d conversion and store it in the appropriate memory slot
                LATAbits.LATA3 = 1; //pulse generated ever 1 ms
                LATAbits.LATA3 = 0;
                adresh_read= (unsigned int)ADRESH;
                adresl_read= (unsigned int)ADRESL;
                cnt_tp++;             //increase the PLED pulse counter

                if (cnt_tp>=1000){
                    LATAbits.LATA6 =~LATAbits.LATA6 ; //pulse PLED
                    cnt_tp=0;                         //reset the counter
                }
                
                //store the conversion
                memory[channel] = ((adresh_read) << 8) | adresl_read;

                //check if either the channel is that of the charging or discharging current and update the coulomb count appropriately

                //for charging current
                if(channel==0x0002){
                    //cast the current reading as a 32 bit int
                    charge_nreading =(uint32_t)(memory[channel]);
                    //update the coulomb count
                    charge_coulomb_count = charge_coulomb_count+charge_nreading;
                    //update the number of charging current reading
                    no_charge_current_reading++;
                }
                //for discharging current
                if(channel==0x0003){
                    //cast the current reading as a 32 bit int
                    discharge_nreading =(uint32_t)(memory[channel]);
                    //update the coulomb count
                    discharge_coulomb_count = discharge_coulomb_count+ discharge_nreading;
                    //update the number of charging current reading
                    no_discharge_current_reading++;
                }


                //update the time elapsed since the by-pass switch has been set
                if(memory[4]==0x0001){
                    tick_ms++;
                    if(tick_ms==1000){
                        tick_ms=0;      //reset the tick_ms counter
                        tick_s++;
                        if(tick_s==60){
                            tick_s=0;   //reset the tick_s counter
                            tick_min++;

                            if(tick_min>=memory[8]){
                               //clear the flag
                                memory[4]=0x0000;
                               //reset the counters
                                tick_ms=0;
                                tick_s=0;
                                tick_min=0;
                            }

                        }

                    }

                }


                //set up the next conversion
                channel++;
                //reset the index, channel if it has been increased to 4
                if(channel==NUM_ADC_CHAN){
                    channel=0;
                }
                //select the channel
                MacroChanSel(channel_select[channel]);

                //clear the ADC interrupt flag
                PIR1bits.ADIF = 0;                      // clear the adc interrupt flag
                cnt_t0=0;                               //reset  the counter
                MacroStartConv();

             }

    }

     if (!loop_start) CLRWDT();
}


void main(){

  _delay(1000);                    //delays for 10 cycles
  OSCCON = 0x72;                   // Switch to fast internal 8 MHz RC Clock

  loop_start=0;
 // Set up port A
  LATA   = 0x00;                   // Pre-clear any outputs
  TRISA= 0b00110011  ;             //set PortA

  //Port Configuration for ADC conversion
  ANSELA= 0b0000011;               //ANSA0-ANSA1-set as analog inputs

  // Set up portB
  LATB   = 0x00;                   // Pre-clear any outputs
  TRISB  = 0b00011110;             //set SDA1(RB1) and SCK1(RB4) as inputs
  
  //Port Configuration for ADC conversion
  ANSELB = 0b00001100;             //ANSB2-ANSB3-set as analog inputs

  /****************************************************************************/

   
  /*Initialize the memory array*/
  memory[0]= 0x0000;                    //stores the cell voltage adc result
  memory[1]= 0x0000;                    //stores the cell temperature adc result
  memory[2]= 0x0000;                    //stores the  pack charging current
  memory[3]= 0x0000;                    //stores the  pack discharging current
  memory[4]= 0x0000;                    //stores the bypass state
  memory[5]= EEPROM_READ(0x00);         //stores the slave ID(default 0x04)
  memory[6]= software_version;          //stores the software version
  memory[7]= 0x0042;                    //stores the test value of 0x0042
  memory[8]= EEPROM_READ(0x01);         //stores the default by-pass time
  memory[9]= 0x0000;                    //the high 16 bits of the coulomb count
  memory[10]=0x0000;                    //the low 16 bits of the coulomb count

  charge_coulomb_count= 0x00000000;
  discharge_coulomb_count= 0x00000000;

  /****************************************************************************/

  /*Set up I2C control registers*/

   SSP1STATbits.SMP = 0 ;          //bit 7 = SMP
   SSP1STATbits.CKE = 1;           //bit 6 = CKE
   SSP1STATbits.BF = 0;            //initially clear the buffer

   //SSP1 Control register1
   SSP1CON1bits.SSPEN = 1;         //enable the serial port and configure the SDA1and
                                   //SCL1 pins
   SSP1CON1bits.CKP = 1;           //releases the clock
   SSP1CON1bits.SSPM= 0b0110;      //I2C Slave mode, 7-bit address

   //SSP1 Control register2
   SSP1CON2bits.GCEN = 0;          //General Call disabled
   SSP1CON2bits.SEN =  1;          /*Clock Stretch for both slave transmit
                                     and receive disabled*/

   //SSP1 Control register3
   SSP1CON3bits.BOEN = 0;         //SSP1BUF is only updated when SSP1OV is clear
   SSP1CON3bits.SDAHT =0;         // 100ns minimum hold time on the SDA1 line
   SSP1CON3bits.SBCDE =0;         //bus collision detection disabled
   SSP1CON3bits.AHEN = 0;         //address holding disabled
   SSP1CON3bits.DHEN = 0;         //data holding disabled

   //SSP1 Mask register
   SSP1MSK= 0xFF;


   //setup Watchdog Timer register
   //WDTCONbits.WDTPS=0b01101 ; //8 s insufficient FOR testing..
   WDTCONbits.WDTPS=0b10000 ;    //Prescale rate 64s)
  WDTCONbits.SWDTEN=1 ;          //WDT is turned on

   //OPTION_REGbits.PS=0b111; //prescale =128 WDT time out =2.3 s
  
  //clear internal watch dog

   CLRWDT();
  
    
   //Slave ID
   SSP1ADD = (char)memory[5];  //Only the upper 7 bits are used for the address, last bit is I2C
                               //read /write. Default address value of 0x04


  
    
/*******************************************************************************/
  /*Configure timer0*/
  OPTION_REGbits.TMR0CS = 0;        //Timer0 clock source- internal instruction cycle clock (FOSC/4)
  OPTION_REGbits.PSA = 1;           //Timer0 Prescale assignment bit- no prescale assigned
/*******************************************************************************/
  /*Configure the ADC module*/

  //Enabling Fixed Voltage Reference Selection
  FVRCONbits.FVREN=1;                 //Fixed voltage reference(FVR) is enabled
  FVRCONbits.ADFVR=0b10;              //ADC Fixed Voltage Reference Peripheral output is(2.048V)

  //Setting the ADC Voltage ref as the FVR
  ADCON1bits.ADNREF=0;                //Vref- is connected to Vss
  ADCON1bits.ADPREF=0b11;             //Vref+ is connectede to the internal FVR module (2.048V)

  //ADC conversion clock source
  ADCON1bits.ADCS= 0b111;             //Frc(8MHz) clock supplied from the dedicated RC oscillator

  //ADC result formatting
  ADCON1bits.ADFM=1;                  //Right justified.

//clear internal watch dog

   //CLRWDT();
  /*****************************************************************************/
 /*Configure the DAC module for an output of 2.048, the FVR, on the Vref pin*/

  FVRCONbits.CDAFVR= 0b10;            //Comparator and DAC Fixed Voltage Reference Peripheral output is 2x (2.048V)

  DACCON0bits.DACEN=0;                //DAC is disabled
  DACCON0bits.DACLPS=1;               //DAC low power voltage state- DAC positive reference source selected
  DACCON0bits.DACOE=1;                //DAC voltage level is also an output on the DACOUT pin
  DACCON0bits.DACPSS=0b10;            //positive power source= FVR buffer output
  DACCON0bits.DACNSS=0;               //negative power source= Vss
  DACCON1bits.DACR=0b11111;           //DAC volatge output select bits

 /*****************************************************************************/


// clear interrupt flags
  INTCONbits.TMR0IF=0;               // the timer interrupt flag
  PIR1bits.SSP1IF = 0;               // the i2c interrupt flag
  PIR1bits.ADIF = 0;                 // the adc interrupt flag

  
//enable interrupts

  INTCONbits.PEIE= 1;                //enable peripheral interrupt
  PIE1bits.SSP1IE = 1;               //enable i2c interrupt
  PIE1bits.ADIE=0;                   //disable ADC interrupts
  INTCONbits.TMR0IE= 1;              //enable timer0 interrupt

  ei();                              //enable global interrupts


// start the first A/D conversion
  MacroChanSel(channel_select[0]);
  MacroStartConv();



  memory[5]=EEPROM_READ(0x00); // assign address from EEPROM
  memory[8]=EEPROM_READ(0x01); //assign bypass time from EEPROM

  loop_start==0;
   
  LATBbits.LATB5=0; 

  while(1){

     
     //if a complete command has been received
      INTCONbits.GIE = 0;
      if(i2c_complete==1){
          
      //clear the flag
          i2c_complete=0;
      //check if a set command
           if((rxByte[1]& 0xF0)==0x00){   
               func_ptr_index= rxByte[1]& 0x0F;
               (*func_ptr[func_ptr_index])(rxByte[2],rxByte[3]);

           }
      }
      INTCONbits.GIE = 1;
      //the bypass switch state LED
      if(memory[4]==0x0001){
        LATBbits.LATB0=1;
      }else{
        LATBbits.LATB0=0;
      }
     
      }

  }

/*Loads the transmission buffer, txByte[] with the appropriate data as per the lower 4 bits of the command number*/
void loadTransBuff(unsigned int command_lower_4){

    if(command_lower_4 <= 0x0008){      //two bytes to be returned

                txByte[0] = ((unsigned char)(((memory[command_lower_4])>>8)& 0x00FF));    //High Byte
                txByte[1] = ((unsigned char)((0x00FF & (memory[command_lower_4]))));      //Low Byte


            }else if(command_lower_4 == 0x0009){                                          //to return charging coulomb count
               //first return the coloumb count
                txByte[0]=  ((unsigned char)(charge_coulomb_count>>24)& 0x000000FF);
                txByte[1] = ((unsigned char)(charge_coulomb_count>>16)& 0x000000FF);
                txByte[2]=  ((unsigned char)(charge_coulomb_count>>8) & 0x000000FF);
                txByte[3] = ((unsigned char)((0x000000FF & charge_coulomb_count )));

                //followed by the number of charging current readings summed to derive the coloumb count

                txByte[4]=  ((unsigned char)(no_charge_current_reading>>24)& 0x000000FF);
                txByte[5] = ((unsigned char)(no_charge_current_reading>>16)& 0x000000FF);
                txByte[6]=  ((unsigned char)(no_charge_current_reading>>8) & 0x000000FF);
                txByte[7] = ((unsigned char)((0x000000FF & no_charge_current_reading )));



            }else if(command_lower_4 == 0x000A){                                          //to return discharging coulomb count

                txByte[0]=  ((unsigned char)(discharge_coulomb_count>>24)& 0x000000FF);
                txByte[1] = ((unsigned char)(discharge_coulomb_count>>16)& 0x000000FF);
                txByte[2]=  ((unsigned char)(discharge_coulomb_count>>8) & 0x000000FF);
                txByte[3] = ((unsigned char)((0x000000FF & discharge_coulomb_count )));

                 //followed by the number of discharging current readings summed to derive the coloumb count

                txByte[4]=  ((unsigned char)(no_discharge_current_reading>>24)& 0x000000FF);
                txByte[5] = ((unsigned char)(no_discharge_current_reading>>16)& 0x000000FF);
                txByte[6]=  ((unsigned char)(no_discharge_current_reading>>8) & 0x000000FF);
                txByte[7] = ((unsigned char)((0x000000FF & no_discharge_current_reading )));

            }else if (command_lower_4 == 0x000B){                                         //return the volt and temp
                //return the volt
                txByte[0]=  ((unsigned char)(((memory[0])>>8)& 0x00FF));
                txByte[1] = ((unsigned char)((0x00FF & (memory[0]))));
                //return the temp
                txByte[2]=  ((unsigned char)(((memory[1])>>8)& 0x00FF));
                txByte[3] = ((unsigned char)((0x00FF & (memory[1]))));


            }else if (command_lower_4 == 0x000C){                                         //return the volt, temp and charging current
               //return the volt
                txByte[0]=  ((unsigned char)(((memory[0])>>8)& 0x00FF));
                txByte[1] = ((unsigned char)((0x00FF & (memory[0]))));
                //return the temp
                txByte[2]=  ((unsigned char)(((memory[1])>>8)& 0x00FF));
                txByte[3] = ((unsigned char)((0x00FF & (memory[1]))));
                //return the charging current
                txByte[4]=  ((unsigned char)(((memory[2])>>8)& 0x00FF));
                txByte[5] = ((unsigned char)((0x00FF & (memory[2]))));

            }else if (command_lower_4 == 0x000D){                                         //return the volt, temp and discharging current
            //return the volt
                txByte[0]=  ((unsigned char)(((memory[0])>>8)& 0x00FF));
                txByte[1] = ((unsigned char)((0x00FF & (memory[0]))));
                //return the temp
                txByte[2]=  ((unsigned char)(((memory[1])>>8)& 0x00FF));
                txByte[3] = ((unsigned char)((0x00FF & (memory[1]))));
                //return the charging current
                txByte[4]=  ((unsigned char)(((memory[3])>>8)& 0x00FF));
                txByte[5] = ((unsigned char)((0x00FF & (memory[3]))));

                }else if (command_lower_4 == 0x000E){                                         //return the time elapsed since the bypass switch has been set
            //return the minute
                txByte[0]=  ((unsigned char)(((tick_min)>>8)& 0x00FF));
                txByte[1] = ((unsigned char)((0x00FF & (tick_min))));
                //return the second
                txByte[2]=  ((unsigned char)(((tick_s)>>8)& 0x00FF));
                txByte[3] = ((unsigned char)((0x00FF & (tick_s))));
                //return the millisecond
                txByte[4]=  ((unsigned char)(((tick_ms)>>8)& 0x00FF));
                txByte[5] = ((unsigned char)((0x00FF & (tick_ms))));


            }else{                                                                        //default

                txByte[0] = 0xEE;
                txByte[1] = 0xEE;
                txByte[2] = 0xEE;
                txByte[3] = 0xEE;
                txByte[4] = 0xEE;
                txByte[5] = 0xEE;
            }
}


/*Gets the voltage of the cell*/
void getVolt(unsigned char x, unsigned char y){}

/*Gets the temperature of the cell*/
void getTemp(unsigned char x, unsigned char y){}

/*Gets the charging current of the pack*/
void getCCurr(unsigned char x, unsigned char y){}

/*Gets the discharging current of the pack*/
void getDCurr(unsigned char x, unsigned char y){}

/*Gets the bypass state- whether it is on(1) or off(0)*/
void getBState(unsigned char x, unsigned char y){}

/*Gets the board/slave pic address*/
void getID(unsigned char x, unsigned char y){}

/*Gets the Coulomb count*/
void getCCount(unsigned char x, unsigned char y){}

/*Gets the software version*/
void getSoftware_Ver(unsigned char x, unsigned char y){}

/*Test Function which should return 0x42(decimal 66)*/
void getHex_42(unsigned char x, unsigned char y){}

/*Gets the time in minutes for which the bypass switch would be on*/
void getBTime(unsigned char x, unsigned char y){}

/*Gets the voltage and temperature of the cell*/
void getVolt_Temp(unsigned char x, unsigned char y){}

/*Gets the voltage, temperature and  charging current of the cell*/
void getVolt_Temp_CCurr(unsigned char x, unsigned char y){}

/*Gets the voltage,temperature and discharging current of the cell*/
void getVolt_Temp_DCurr(unsigned char x, unsigned char y){}


/*Sets the bypass resistor switch state*/
void setBState(unsigned char x, unsigned char y){
    if (y==0x01){
    memory[4]=0x0001;
    }
    else {
        memory[4]=0x0000;
         tick_ms = 0;        // reset count number of millisecond
         tick_s = 0;         // reset count number of seconds
         tick_min = 0;       // reset count number of minutes
    }
    }


/*Sets the slave address */
void setID(unsigned char x, unsigned char y){
    /*write in EEPROM the address 00
     * x=Data high byte received
     * y=Data low byte received
     * Address is lower 7 bits of low byte so x technically 00
     * 
     */

    EEPROM_WRITE(0x00,y);
    while(WR) continue;
    memory[5]=EEPROM_READ(0x00); //re assign memory.
    SSP1ADD = (char)memory[5];  //re assign SSP1ADD
   }

/*Sets the Bypass time in minutes*/
void setBTime(unsigned char x, unsigned char y){
/*write in EEPROM the address 01
     * x=Data high byte received
     * y=Data low byte received
     * Address is lower 7 bits of low byte so x technically 00
     *
     */
    EEPROM_WRITE(0x01,y);
    while (WR) continue;
    memory[8]=EEPROM_READ(0x01); //reassign memory[8]
}

/*starts a loop in I2C interrupt*/
void startLoop (unsigned char x, unsigned char y){

   /*assign a value of 1 to a variable in the function
    which starts a forever loop and causes WDT to overflow*/
    loop_start=1;
}

